import apache_beam as beam
from apache_beam.options.pipeline_options import PipelineOptions


class Experiments:
    def __init__(self):
        self.models = []
        self.metrics = []
        self.train_datasets = []
        self.test_datasets = []
        self.avro_schema = {}
        self.steps = []

    def add_model(self, model):
        self.models.append(model)

    def add_metric(self, metric):
        self.metrics.append(metric)

    def add_train(self, train_dataset):
        self.train_datasets.append(train_dataset)

    def add_test(self, test_dataset):
        self.test_datasets.append(test_dataset)

    def add_step(self, step):
        self.steps.append(step)

    def run(self, initial_input = None):

        print("===================================")
        print("Starting the experiment pipeline...")
        print("===================================")
        
        experiments = {
            "models": self.models,
            "train_datasets": self.train_datasets,
            "test_datasets": self.test_datasets,
            "metrics": self.metrics,
        }

        if initial_input:
            experiments.update(initial_input)

        pipeline_options = PipelineOptions()
        with beam.Pipeline(options=pipeline_options) as p:

            experiments = (
                p 
                | "Create Experiments" >> beam.Create([experiments])
            )

            for step in self.steps:  
                experiments = (
                    experiments 
                    | f"Custom Step: {step.name()}" >> beam.ParDo(step)
                )

        print("=============================")
        print("Experiment pipeline finished.")
        print("=============================")





from pathlib import Path
from atelierflow.datasets.dataset import Dataset


class AcousticDataset(Dataset):
  def __init__(self, root_dir, pattern = ".wav", include_abnormal = True, include_normal = True):
    """
    Args:
        root_dir (str): Caminho para o diretório raiz que contém os arquivos de áudio.
        include_abnormal (bool): Indica se os dados coletados devem incluir dados "anormais".
        include_normal (bool): Indica se os dados coletados devem includir dados "normais".
        pattern (str): Indica qual arquivo procurar.
        Estrutura esperada:
          root_dir/
              ├── machine_type_1/
              │   ├── id_00/
              │   │   ├── abnormal/
              │   │   └── normal/
              │   └── id_01/
              └── machine_type_2/
                  ├── id_00/
                  └── id_01/
    """
    self.paths = []  
    self.labels = [] 
    self.include_abnormal = include_abnormal
    self.include_normal = include_normal
    self.pattern = pattern  
    self._collect_files(root_dir)

  def _collect_files(self, root_dir):
    """
    Percorre o diretório root_dir para coletar paths dos arquivos de áudio
    e gerar rótulos com base nas pastas 'normal' e 'abnormal'.
    """
    for machine_dir in Path(root_dir).iterdir():
      if machine_dir.is_dir():
        for id_dir in machine_dir.iterdir():
          if id_dir.is_dir():
            
            if self.include_normal:
              normal_dir = id_dir / 'normal'
              if normal_dir.exists():
                for audio_file in normal_dir.glob(f"*{self.pattern}"):  
                  self.paths.append(str(audio_file))
                  self.labels.append(0)  

         
            if self.include_abnormal:
              abnormal_dir = id_dir / 'abnormal'
              if abnormal_dir.exists():
                for audio_file in abnormal_dir.glob(f"*{self.pattern}"):  
                  self.paths.append(str(audio_file))
                  self.labels.append(1)  

    if not self.paths:
      print(f"[Error] No matching files were found in '{root_dir}' with pattern '{self.pattern}'")

  def __getitem__(self, index):
    audio_path = self.paths[index]
    label = self.labels[index]
    return audio_path, label

  def __len__(self):
    return len(self.paths)








from abc import ABC, abstractmethod

class BaseMetric(ABC):
  def __init__(self):
    raise NotImplementedError("Subclasses must implement this method.")

  @abstractmethod
  def compute(self):
    raise NotImplementedError("Subclasses must implement this method.")


import numpy as np

class DataLoader:
    def __init__(self, dataset, batch_size=8, shuffle=None):
        self.dataset = dataset
        self.batch_size = batch_size
        self.shuffle = shuffle
        self.indices = np.arange(len(dataset))  
        self.current_idx = 0                   

    def __len__(self):
        return int(np.ceil(len(self.dataset) / self.batch_size))

    def __iter__(self):
        self.current_idx = 0
        if self.shuffle:
            np.random.shuffle(self.indices)
        return self

    def __next__(self):
        if self.current_idx >= len(self.dataset):
            raise StopIteration
        
        batch_indices = self.indices[self.current_idx:self.current_idx + self.batch_size]
        batch = [self.dataset[i] for i in batch_indices] 

        self.current_idx += self.batch_size
        
        X_batch, y_batch = zip(*batch)
        
        return np.array(X_batch), np.array(y_batch)


